unit Project;

interface
uses IniFiles, SysUtils, Dialogs, windows, ComCtrls, MDIEDit, Utils, Classes, Controls;

var ProjectOpen : Boolean;

const MAXUNIT = 400;   // Maximum number of units in a project

type TUnit =  record
                FileName : string;
                Node     : TTreeNode;
                Modified : boolean;
                FileTime : LongInt;
              end;

type TProject = class;

     TProject = class(TObject)
     constructor Create(nFileName, nName : string);
     procedure Free;
     function AddUnit(nFileName : string; OpenProject : boolean; CreateNodes : boolean;
              ItsNewProject : Boolean) : TEditForm;
     function  FileAlreadyExists(s : string) : boolean;
     procedure CreateNewResFile(nFileName : string);
     function ShowUnit(nFileName : string): TEditForm;
     function AllObjFilesExist : boolean;
     function GetUnit(s : string) : TUnit;
     function GetUnitIndex(s : string) : integer;
     procedure SetMainUnit(s : string);

     public
       Name      : string;
       FileName  : string;
       Units     : array [0..MAXUNIT] of TUnit;
       UnitCount : integer;
       Icon      : string;
       ObjFile   : string;
       CompilerOptions : string;
       IncludeDirs : string;
       MainUnit : string;

       ResFiles  : string;
       ResFileModified : boolean;

       NoConsole : boolean;
       IsDll     : boolean;

     private
       IniFile : TIniFile;
       CountUnitCreated : integer;
     end;

Function ToFrontSlash (Const s : String) : String;

implementation
uses Main, NewProject;

constructor TProject.Create(nFileName, nName: string);
var i, tmpCount : integer;
    tmp : string;
begin
  inherited Create;
  FileName := nFileName;
//ShowMessage (filename+'=='+nname);
  MainForm.CloseAll;

  IniFile := TIniFile.Create(Filename);
  ProjectOpen := True;

  IniFile.WriteString('Project','FileName',FileName);

  if nName = '__FileOpen' then
  begin
    Name := IniFile.ReadString('Project','Name',Name);
    MainForm.ProjectNode := MainForm.MakeProjectNode(Name);
    UnitCount := IniFile.ReadInteger('Project','UnitCount',UnitCount);
    IsDll := IniFile.ReadBool('Project','IsDll', False);

    tmpCount := UnitCount;
    UnitCount := 0;
    for i := 0 to tmpCount-1
    do begin
        tmp := IniFile.ReadString('Unit'+IntToStr(i+1),'FileName',tmp);
        if not FileExists(tmp)
        then begin
           if FileExists(ExtractFilePath(FileName)+ExtractFileName(tmp))
           then begin
              IniFile.WriteString('Unit'+IntToStr(i+1),'FileName',ExtractFilePath(FileName)+ExtractFileName(tmp));
              AddUnit(ExtractFilePath(FileName)+ExtractFileName(tmp),True, True, False);
           end
           else begin
              MessageDlg('File '+tmp+' not found. Please check your unit filenames in Project menu.',MtError,[mbOK],0);
              AddUnit(tmp, True, True, False);
           end
        end
        else AddUnit(tmp, True, True, False);
    end;
    ObjFile := IniFile.ReadString('Project','ObjFile',ObjFile);

    ResFiles := IniFile.ReadString('Project','ResFiles',ResFiles);

    CompilerOptions := IniFile.ReadString('Project', 'CompilerOptions', '');
    IncludeDirs := IniFile.ReadString('Project', 'IncludeDirs', '');
    NoConsole := IniFile.ReadBool('Project','NoConsole',NoConsole);
    Icon := IniFile.ReadString('Project','Icon','Mainicon.ico');
    MainUnit := IniFile.ReadString('Project','MainUnit','');
  end
  else begin
    Name := nName;
    MainForm.ProjectNode := MainForm.MakeProjectNode(Name);
    IniFile.WriteString('Project','Name',Name);

    UnitCount := 0;

//    showmessage (nfilename+'=='+name+'===='+nname);
    // New project being created
    AddUnit(ChangeFileExt (ExtractFileName (nFileName), ''){'Untitled'}, False, True, True);
    Name := nName;

    ResFiles := '';
    If ThisCompiler <> GPC then ResFiles := ExtractFilePath (nFileName)+ 'rsrc.rc';
    IniFile.WriteString ('Project', 'ResFiles', ResFiles);

    If ThisCompiler <> GPC then
      if not FileExists(ExtractFilePath(nFilename) + 'rsrc.rc') then
         CreateNewResFile(MainForm.DevPasPath + 'Icon\MAINICON.ICO');

    ObjFile := '';

    NoConsole := not NewProjForm.Console;

    if (ProjectType = TDll) then
        IsDll := True
    else
        IsDll := False;

    IniFile.WriteBool('Project','NoConsole',NoConsole);
    IniFile.WriteBool('Project','IsDll',IsDll);
    IniFile.WriteString('Project','Icon',MainForm.BinDir+'Mainicon.ico');
    Icon := MainForm.BinDir+'Mainicon.ico';
  end;

  if nName = '__FileOpen' then
     Name := IniFile.ReadString('Project','Name',Name);

  FileName := nFileName;

  MainForm.SaveFileHistory(FileName);
end;

procedure TProject.Free;
begin
  ProjectOpen := False;
  ProjectType := TOther;
end;

Function ToFrontSlash (Const s : String) : String;
Var j : integer;
Begin
  Result := s;
  For j := 1 to length (Result) do if Result [j] = '\' then Result [j] := '/';
End;

procedure TProject.CreateNewResFile(nFileName : string);
var F1 : TextFile;
    s : string;
begin
  AssignFile(F1,ExtractFilePath(FileName)+'Rsrc.rc');
  Rewrite(F1);
  s := ToFrontSlash (nFileName);
  Writeln(F1,'500 ICON MOVEABLE PURE LOADONCALL DISCARDABLE "' + s + '"');
  CloseFile(F1);
  ResFileModified := True;
end;

Function NoExt (Const s:String):String;
begin
   Result := ChangeFileExt (s, '');
end;
(*
Function GetMainProgramForm : TEditForm;
Var
i: Word;
e : TEditForm;
Begin
 Result := Nil;
 for i := 0 to MainForm.MDIChildCount - 1
 do begin
     e := TEditForm (MainForm.MDIChildren[i]);
     If (e.Pathname <> '') and (e.GetNamePath = 'EditForm')
     then begin
        Result := e;
        Exit;
     end;
     //Showmessage (inttostr(i)+'='+e.pathname+'=='+mainform.project.FileName);
 end;
End;
*)
function TProject.AddUnit;//(nFileName : string; OpenProject : boolean; CreateNodes : boolean) : TEditForm;
var
s, s1, ProjectName: string;
i, j : Integer;
e : TEditForm;
begin
  Inc(UnitCount);

  if nFileName = NewUnitTitle
  then begin
     {  4/4/04 find out whether other "Unit[n]" units exist in the project }
     Inc (CountUnitCreated);
     For i := 1 to UnitCount
     do begin
        // showmessage (Units[i].FileName+'==='+nFilename);
         s1 :=  ExtractFileName (Units[i].FileName);
         //showmessage (NoExt (s1) +'==='+nFilename+ IntToStr(CountUnitCreated));
         If AnsiCompareText (NoExt (s1), nFileName + IntToStr(CountUnitCreated)) = 0
         then begin
            //showmessage (NoExt(ExtractFileName (Units[i].FileName)));
             Inc (CountUnitCreated);
         end;
     end;
     nFileName := NewUnitTitle + IntToStr(CountUnitCreated);
  end;

  if UnitCount = 1 then
     SetMainUnit(nFileName);

  Units[UnitCount-1].FileName := nFileName;
  Units[UnitCount-1].Modified := False;
  Units[UnitCount-1].FileTime := IniFile.ReadInteger('Unit'+IntToStr(UnitCount),'FileTime',0);

  if OpenProject = False then
     IniFile.WriteString('Unit'+IntToStr(UnitCount),'FileName',nFileName);

  IniFile.WriteInteger('Project','UnitCount',UnitCount);

  if CreateNodes then
     Units[UnitCount-1].Node := MainForm.MakeNewFileNode(nFileName);

  if OpenProject and MainForm.IniFile.ReadBool('Options','NoOpen', False)
  then begin
     Result := nil;
     exit;
  end;

  Result := TEditForm.Create(MainForm);

  ProjectName := Name;
  if Pos(':\',nFileName) <> 0
  then begin
     with Result
     do begin
       Caption := ExtractFileName(nFileName);
       PathName := nFileName;
       try
         Editor.Lines.LoadFromFile(nFileName);
       except
         Messagedlg('Could not open file '+nFileName, mtWarning, [mbOK],0);
       end;

       GetIndex := UnitCount;
     end
  end
  else
    with Result
    do begin
       Caption := ExtractFileName(nFileName);
       PathName := '';
       GetIndex := UnitCount;

       Editor.CaretX := 0;
       if  ((nFileName = NewUnitTitle + IntToStr(CountUnitCreated)) and (NewProjForm <> nil)
       or (ItsNewProject))
       and (NewProjForm.ModalResult = mrOK)
       then begin
          Editor.Lines.Text := NewProjForm.Text;
          if NewProjForm.X < 300
          then begin
             Editor.CaretX := NewProjForm.X;
             Editor.CaretY := NewProjForm.Y;
          end;
       end;

       //If Pos ('untitled', lowercase (nFilename)) > 0
       {If Pos ('unit', lowercase (nFilename)) > 0
       then begin
          Editor.Lines.Text := NewProjForm.Text
       end
       else begin
       }
       If Editor.Lines.Count = 0 then With Editor.Lines
       do begin
           If ItsNewProject
           then begin
               Add ('Program ' + nFileName + ';');
               Add ('');
               Add ('Uses Messages, SysUtils, Windows;');
               Add ('');
               Add ('Begin');
               Add ('End.');
           end
           else begin
               Add ('Unit ' + nFileName + ';');
               Add ('');
               Add ('Interface');
               Add ('Uses Messages, SysUtils, Windows;');
               Add ('');
               Add ('Implementation');
               Add ('');
               Add ('End.');

               // try and add to the USES clause
               e := GetMainProgramForm;
               If e <> Nil then With e
               do begin
                 With Editor
                 do begin
                   for i := 0 to Pred (Lines.Count)
                   do begin
                     s := Trim (Lines [i]);
                     j := Pos ('USES', Uppercase (s));
                     If j > 0 then begin
                        If  (Pos ('USES ', Uppercase (s)) = j) // units on same line
                        and (s [Length (s)] in [',', ';'])
                        then begin
                           s1 := nFileName;
                           If s [Length (s)] = ',' // line ends with comma
                           then begin
                              s1 := s1 + ',';
                              s := s + ' ' + s1;
                           end  // else line ends with semi-colon
                           else System.Insert (', ' + s1, s, Length (s){j+5});
                           Lines [i] := s; // insert on same line
                        end // else "USES" on line by itself - insert unit on new line
                        else Lines.Insert (i + 1, nFileName + ', ');
                        Break;
                     end;
                   end;
                 end;
           end;

           end;
       end;
     {end;}
  end;
end;

function TProject.FileAlreadyExists(s : string) : boolean;
var i : integer;
begin
  Result := False;
  for i := 0 to UnitCount-1 do
      if lowercase(ExtractFileName(s)) = lowercase(ExtractFileName(Units[i].FileName)) then
         Result := True;
end;

function TProject.ShowUnit(nFileName : string) : TEditForm;
var i : integer;
begin
  Result := TEditForm.Create(MainForm);
  with Result do begin
    Caption := ExtractFileName(nFileName);

    if pos(':\',nFileName) = 0 then
       exit;

    PathName := nFileName;

    for i := 0 to UnitCount-1 do
        if lowercase(ExtractFileName(nFileName)) = lowercase(ExtractFileName(Units[i].FileName)) then begin
           GetIndex := i;
           Open(Units[i].FileName);

           exit;
        end;
  end
end;

function TProject.AllObjFilesExist : boolean;
var i : integer;
begin
  Result := True;
  for i := 0 to UnitCount-1 do
      if FileExists(ChangeFileExt(GetShortName(Units[i].FileName),'.ow')) then begin
           Result := False;
           Exit;
      end;
end;

function TProject.GetUnit(s : string) : TUnit;
var i : integer;
begin
  for i := 0 to UnitCount-1 do
      if lowercase(s) = lowercase(ExtractFileName(Units[i].FileName)) then begin
         Result := Units[i];
         exit;
      end;
end;

function TProject.GetUnitIndex(s : string) : integer;
var i : integer;
begin
  Result := -1;
  for i := 0 to UnitCount-1 do
      if lowercase(s) = lowercase(ExtractFileName(Units[i].FileName))
      then begin
         Result := i;
         exit;
      end;
end;

procedure TProject.SetMainUnit(s : string);
begin
  MainUnit := s;
  IniFile.WriteString('Project','MainUnit',s);
end;

end.
